# _*_coding:utf-8_*_

from django.apps import AppConfig


class ResourceConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'xj_resource'
    verbose_name = u'Ⅵ 资源系统'
    sort = 6
